function c = invijk(a, F)
% CONJ   Quaternion perperndicular involution.
% (Quaternion overloading of Matlab standard function.)
%
% Implements three perpendicular involutions:
%
% conj(X) or
% conj(X, 'i') returns the i-involution
% conj(X, 'j')  returns the j-involution
% conj(X, 'k')    returns the k-involution

% Copyright  2010 Clive Cheong Took
% See the file : Copyright.m for further details.

error(nargchk(1, 2, nargin)), error(nargoutchk(0, 1, nargout))

if nargin == 1
    F = 'i'; % Supply the default parameter value.
end

if ~strcmp(F, 'i') && ~strcmp(F, 'j') && ~strcmp(F, 'k')
    error('Second parameter value must be ''i'', ''j'' or ''k''.')
end

switch F
    case 'i'
            c = class(compose(ess(a), ...
                              exe(a), -wye(a), -zed(a)), 'quaternion');
        
    case 'j'
        
            c = class(compose(ess(a), ...
                             -exe(a),  wye(a), -zed(a)), 'quaternion');
        
    case 'k'
            c = class(compose(ess(a), ...
                             -exe(a), -wye(a), zed(a)), 'quaternion');
        
    otherwise
        error('Bad value for second parameter.');
end
