%  Example to generate second order quaternion statistics in the paper:
% C. Cheong Took and D. P. Mandic, Augmented Second-Order Statistics of 
% Quaternion Random Signals, Signal Processing, vol. 91, no. 2, pp. 214-224, 
% 2011; 
% 
% 
%  OUTPUT:
%  C_i: i-covariance matrix in equation (14)
%  C_j: j-covariance matrix in equation (15)
%  C_k: k-covariance matrix in equation (16)
% 
% 
%  
%(c) Copyright  Clive Cheong Took 2013 (c.cheongtook@surrey.ac.uk).
%https://www.surrey.ac.uk/computing/research/nice/people/clive_cheong_took/
%  
%  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     This program is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 2 of the License, or
%     (at your option) any later version.
%  
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
%  
%     You can obtain a copy of the GNU General Public License from
%     http://www.gnu.org/copyleft/gpl.html or by writing to
%     Free Software Foundation, Inc.,675 Mass Ave, Cambridge, MA 02139, USA.
%  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This implementation uses a "modified" version of the Quaternion toolbox   
% for Matlab available at http://qtfm.sourceforge.net/. The "modified"
% version is included in this package for the purpose of simulation on
% quaternion signal processing by Clive Cheong Took.
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% We set a simple 2 x 2 i-covariance, j-covariance, k-covariance matrices
clear all, clc
N=1000;
x1=randn(8,N);
%Zero mean data
x1=x1-repmat(mean(x1,2),1,N);
[U, D]=eig((x1*x1')/N);
x1=D^(-0.5)*U'*x1;
Xc=[1 0.9 0.85 0.8 0.75 0.7 0.65 0.9];
Xc=transpose(Xc)*Xc;
Xc=Xc-diag(diag(Xc))+diag(ones(1,8));
[U,S,V] = svd(Xc);
x1=U*(S^0.5)*x1;
clear('S','U')
%Initialise the q)uaternion data
q=quaternion(x1(1:2,:),x1(3:4,:),x1(5:6,:),x1(7:8,:) );

%Calculate the quaternion covariance matrices
C_i=(q*invijk(q,'i')')/N % i-covariance matrix
C_j=(q*invijk(q,'j')')/N % j-covariancematrix
C_k=(q*invijk(q,'k')')/N % k-covariance matrix

%Display i-covariance
icovariance11 = C_i(1,1)
icovariance12 = C_i(1,2)
icovariance21 = C_i(2,1)
icovariance22 = C_i(2,2)











