function [U,S] = isvd(X, F)
% SVD    i-Singular value decomposition.
%Use SVD [U,S,V]=svd(X) to perform the diagonalisation of the i-Hermitian
%matrix X
%
%If F= 'U', then the factorisation is (U)D(U^iH) where D is a diagonal
%matrix with vanishing of i-component values .
%
%If F= 'V', then the factorisation is (U^i)D(U^H) where D is a diagonal
%matrix with vanishing of i-component values .
% (Quaternion overloading of standard Matlab function.)

% Copyright  2010 Clive Cheong Took
% See the file : Copyright.m for further details.


error(nargchk(1, 2, nargin)), error(nargoutchk(0, 2, nargout))

if nargin == 1
    F = 'U'; % Supply the default parameter value.
end

if ~strcmp(F, 'U') && ~strcmp(F, 'V') 
    error('Second parameter value must be ''U'' or ''V''.')
end

[U1,S1,V1]=svd(X);

switch F
    case 'U'
           U=U1;
           Qi=mtimes(ctranspose(V1),invijk(U1,'i'));
           S=mtimes(S1,Qi);
        
    case 'V'
            U=V1;
            Q=mtimes(ihermitian(V1),U1);
            S= mtimes(Q,S1);
                    
    otherwise
        error('Bad value for second parameter.');
end
