%  Example to simulate Quaternion Least Mean Square algorithm on Saito
%  Circuit  Attractor
%  C. Cheong Took and D. P. Mandic, A Quaternion Widely Linear Adaptive Filter, 
% IEEE Transactions on Signal Processing, vol. 58, no. 8, pp. 4427 - 4431, 2010; 
% 
%  INPUT:
%  x1: input signal 
%  l: filter length
%  mu: step-size
% 
%  OUTPUT:
%  y1: filter output
% 
% 
%  
%(c) Copyright  Clive Cheong Took 2013 (c.cheongtook@surrey.ac.uk).
%https://www.surrey.ac.uk/computing/research/nice/people/clive_cheong_took/
%  
%  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     This program is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 2 of the License, or
%     (at your option) any later version.
%  
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
%  
%     You can obtain a copy of the GNU General Public License from
%     http://www.gnu.org/copyleft/gpl.html or by writing to
%     Free Software Foundation, Inc.,675 Mass Ave, Cambridge, MA 02139, USA.
%  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This implementation uses a "modified" version of the Quaternion toolbox   
% for Matlab available at http://qtfm.sourceforge.net/. The "modified"
% version is included in this package for the purpose of simulation on
% quaternion signal processing by Clive Cheong Took.
%  

clear all,close all


%%%%%%%%%%%%%%%%%%%%%%%%SAITOS CIRCUIT%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Saitos Circuit
%Torus Doubling and Hyperchaos in a five dimensional hysteresis circuit,
%Proc. of 1994 IEEE Int. Symposium on circuit and systems, vol 6 pp.
%113-116, London
time=10;
[t,x11] = ode45('saito',[0 time],[1;0;1;0]);

no_sig=4; % no of dimension of data
L=length(x11); % length of data
N=length(x11)-1 ; %For training	
x1=(quaternion(x11(:,1),x11(:,2),x11(:,3),x11(:,4)));
d=x1;
l=5;
y1 = transpose(quaternion(zeros(1,L),zeros(1,L),zeros(1,L),zeros(1,L)));
e=y1;
w = transpose(quaternion(zeros(1,l),zeros(1,l),zeros(1,l),zeros(1,l)));
m = w;
gg=w;
hh=w;


%begin of algorithm
for n = l : N
        
	u = x1(n:-1:n-l+1) ;
    mu=1*10^-4;
    
    %widely linear model
    y1(n+1)= sum(times(conj(w),u))+ ...
        + sum(times(conj(gg),(invijk(u,'i'))))+ sum(times(conj(hh),(invijk(u,'j'))))+...
        sum(times(conj(m),(invijk(u,'k'))));
    
    e(n) = d(n+1) - y1(n+1) ;  
    
    %UPDATES OF WEIGHTS
    %Standard part 
 	w = w + mu *( 2*times(u,conj(e(n))) - times(e(n),conj(u)) ) ;
    
%     % Augmented CONJUGATE part, see footnote 2 in paper
%     m = m + mu *( 2*times(conj(u),conj(e(n))) - times(e(n),conj(u)) ) ;
  
%Involutions part weight updates u^i and u^j and u^k
     gg = gg + mu *( 2*times(invijk(u,'i'),conj(e(n))) - times(e(n),conj(invijk(u,'i'))) ) ;
     hh = hh + mu *( 2*times(invijk(u,'j'),conj(e(n))) - times(e(n),conj(invijk(u,'j'))) ) ;
     m =   m + mu *( 2*times(invijk(u,'k'),conj(e(n))) - times(e(n),conj(invijk(u,'k'))) ) ;
    
    
% Haykin way Hermitian    
%     y1(n+stepAhead)= sum(times(conj(w),u))+ sum(times(conj(m),conj(u)));
%     e(n) = d(n+stepAhead) - y1(n+stepAhead) ;            
%  	w = w + mu *( 2*times(u,conj(e(n))) - times(e(n),u) ) ;
%     m = m + mu *( 2*times(conj(u),conj(e(n))) - times(e(n),conj(u)) ) ;
    
% Widrow way Transpose    
%     y1(n+stepAhead)= sum(times(w,u))+ sum(times(m,conj(u)));
%     e(n) = d(n+stepAhead) - y1(n+stepAhead) ;            
%  	w = w + mu *( 2*times(e(n),conj(u)) - times(conj(u),conj(e(n))) ) ;
%     m = m + mu *( 2*times(e(n),u) - times(u,conj(e(n))) ) ;
    

end 



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
start=1;
endd=length(x1);

figure(1)

subplot(411),plot(scalar(x1(start:endd)),'r')
hold on,plot(scalar(y1(start:endd)),'b-.'),ylabel('X')

subplot(412),plot(x(x1(start:endd)),'r')
hold on,plot(x(y1(start:endd)),'b-.'),ylabel('X')

subplot(413),plot(y(x1(start:endd)),'r')
hold on,plot(y(y1(start:endd)),'b-.'),ylabel('Y')

subplot(414),plot(z(x1(start:endd)),'r')
hold on,plot(z(y1(start:endd)),'b-.'),ylabel('Z')
legend('Actual','Estimated')


