function xdot = saito(t,x)
%Saitos Circuit
%Torus Doubling and Hyperchaos in a five dimensional hysteresis circuit,
%Proc. of 1994 IEEE Int. Symposium on circuit and systems, vol 6 pp.
%113-116, London
xdot = zeros(4,1);

eta    = 1.3;
alpha1 = 7.5;
alpha2 = 15;
beta1  = 0.16;
beta2  = 0.097;

rho1= beta1/(1-beta1);
rho2= beta2/(1-beta2);

z  = x(1)+x(3);
if z>=-1
    hz = 1;
elseif z<=1
    hz=-1;
end

xdot(1) = -( x(1)- eta*rho1*hz) + (x(2)- eta*rho1*hz/beta1 );
xdot(2) = -alpha1*( x(1)- eta*rho1*hz) + alpha1*beta1*(x(2)- eta*rho1*hz/beta1 );
xdot(3) = -( x(3)- eta*rho2*hz) + (x(4)- eta*rho2*hz/beta2 );
xdot(4) = -alpha2*( x(3)- eta*rho2*hz) + alpha2*beta2*(x(4)- eta*rho2*hz/beta2 );